from http import HTTPStatus
from typing import Any, Dict

import httpx

from gigachat.api.utils import build_headers
from gigachat.exceptions import AuthenticationError, ResponseError
from gigachat.models import Token


def _get_kwargs(
    *,
    user: str,
    password: str,
) -> Dict[str, Any]:
    headers = build_headers()

    return {
        "method": "POST",
        "url": "/token",
        "auth": (user, password),
        "headers": headers,
    }


def _build_response(response: httpx.Response) -> Token:
    if response.status_code == HTTPStatus.OK:
        return Token(**response.json())
    elif response.status_code == HTTPStatus.UNAUTHORIZED:
        raise AuthenticationError(response.url, response.status_code, response.content, response.headers)
    else:
        raise ResponseError(response.url, response.status_code, response.content, response.headers)


def sync(
    client: httpx.Client,
    *,
    user: str,
    password: str,
) -> Token:
    kwargs = _get_kwargs(user=user, password=password)
    response = client.request(**kwargs)
    return _build_response(response)


async def asyncio(
    client: httpx.AsyncClient,
    *,
    user: str,
    password: str,
) -> Token:
    kwargs = _get_kwargs(user=user, password=password)
    response = await client.request(**kwargs)
    return _build_response(response)



from typing import Any, Dict, Optional

from gigachat.models.function_parameters import FunctionParameters
from gigachat.pydantic_v1 import BaseModel


class Function(BaseModel):
    """Функция, которая может быть вызвана моделью"""

    name: str
    """Название функции"""
    description: Optional[str] = None
    """Описание функции"""
    parameters: Optional[FunctionParameters] = None
    """Список параметров функции"""
    return_parameters: Optional[Dict[Any, Any]] = None
    """Список возвращаемых параметров функции"""



from typing import Any, Dict, List, Optional

from gigachat.pydantic_v1 import BaseModel, Field


class FunctionParametersProperty(BaseModel):
    """Функция, которая может быть вызвана моделью"""

    type_: str = Field(default="object", alias="type")
    """Тип аргумента функции"""
    description: str = ""
    """Описание аргумента"""
    items: Optional[Dict[str, Any]] = None
    """Возможные значения аргумента"""
    enum: Optional[List[str]] = None
    """Возможные значения enum"""
    properties: Optional[Dict[Any, "FunctionParametersProperty"]] = None



from gigachat.pydantic_v1 import BaseModel, Field


class TokensCount(BaseModel):
    """Информация о количестве токенов"""

    tokens: int
    """Количество токенов в соответствующей строке."""
    characters: int
    """Количество токенов в соответствующей строке."""
    object_: str = Field(alias="object")
    """Тип сущности в ответе, например, список"""



from contextvars import ContextVar
from typing import Optional

authorization_cvar: ContextVar[Optional[str]] = ContextVar("authorization_cvar", default=None)
"""Информация об авторизации с помощью JWE"""
client_id_cvar: ContextVar[Optional[str]] = ContextVar("client_id_cvar", default=None)
"""[DEPRECATED] Уникальный ID клиента"""
request_id_cvar: ContextVar[Optional[str]] = ContextVar("request_id_cvar", default=None)
"""Уникальный ID запроса"""
session_id_cvar: ContextVar[Optional[str]] = ContextVar("session_id_cvar", default=None)
"""Уникальный ID сессии"""
service_id_cvar: ContextVar[Optional[str]] = ContextVar("service_id_cvar", default=None)
operation_id_cvar: ContextVar[Optional[str]] = ContextVar("operation_id_cvar", default=None)



from typing import List

from gigachat.models.embeddings_usage import EmbeddingsUsage
from gigachat.pydantic_v1 import BaseModel, Field


class Embedding(BaseModel):
    """Ответ модели"""

    embedding: List[float]
    """Эмбеддинг"""
    usage: EmbeddingsUsage
    """Данные об использовании модели"""
    index: int
    """Индекс эмбеддинга в массиве"""
    object_: str = Field(alias="object")
    """Название объекта"""



from typing import List, Optional

from gigachat.models.embedding import Embedding
from gigachat.pydantic_v1 import BaseModel, Field


class Embeddings(BaseModel):
    """Ответ модели"""

    data: List[Embedding]
    """Массив ответов эмбеддера"""
    model: Optional[str] = None
    """Название модели с помощью которой нужно вычислить эмбеддинги"""
    object_: str = Field(alias="object")
    """Название вызываемого метода"""



from typing import Dict, Optional

from gigachat.context import (
    authorization_cvar,
    operation_id_cvar,
    request_id_cvar,
    service_id_cvar,
    session_id_cvar,
)

USER_AGENT = "GigaChat-python-lib"


def build_headers(access_token: Optional[str] = None) -> Dict[str, str]:
    headers = {}

    if access_token:
        headers["Authorization"] = f"Bearer {access_token}"

    headers["User-Agent"] = USER_AGENT

    authorization = authorization_cvar.get()
    session_id = session_id_cvar.get()
    request_id = request_id_cvar.get()
    service_id = service_id_cvar.get()
    operation_id = operation_id_cvar.get()

    if authorization:
        headers["Authorization"] = authorization
    if session_id:
        headers["X-Session-ID"] = session_id
    if request_id:
        headers["X-Request-ID"] = request_id
    if service_id:
        headers["X-Service-ID"] = service_id
    if operation_id:
        headers["X-Operation-ID"] = operation_id
    return headers



from typing import Any, Dict, Optional

from gigachat.pydantic_v1 import BaseModel


class FunctionCall(BaseModel):
    """Вызов функции"""

    name: str
    """Название функции"""
    arguments: Optional[Dict[Any, Any]] = None
    """Описание функции"""



from typing import Optional

from gigachat.pydantic_v1 import BaseSettings

ENV_PREFIX = "GIGACHAT_"

BASE_URL = "https://gigachat.devices.sberbank.ru/api/v1"
AUTH_URL = "https://ngw.devices.sberbank.ru:9443/api/v2/oauth"
SCOPE = "GIGACHAT_API_PERS"


class Settings(BaseSettings):
    base_url: str = BASE_URL
    """Адрес относительно которого выполняются запросы"""
    auth_url: str = AUTH_URL
    """Адрес для запроса токена доступа OAuth 2.0"""
    credentials: Optional[str] = None
    """Авторизационные данные"""
    scope: str = SCOPE
    """Версия API, к которой предоставляется доступ"""
    access_token: Optional[str] = None
    """JWE токен"""
    model: Optional[str] = None
    """Название модели, от которой нужно получить ответ"""
    profanity_check: Optional[bool] = None
    """Параметр цензуры"""

    user: Optional[str] = None
    password: Optional[str] = None

    timeout: float = 30.0
    verify_ssl_certs: bool = True

    verbose: bool = False

    ca_bundle_file: Optional[str] = None
    cert_file: Optional[str] = None
    key_file: Optional[str] = None
    key_file_password: Optional[str] = None

    class Config:
        env_prefix = ENV_PREFIX



from typing import Any, Dict, List, Optional

from gigachat.models.function_paramaers_property import FunctionParametersProperty
from gigachat.pydantic_v1 import BaseModel, Field


class FunctionParameters(BaseModel):
    """Функция, которая может быть вызвана моделью"""

    type_: str = Field(default="object", alias="type")
    """Тип параметров функции"""
    properties: Optional[Dict[Any, FunctionParametersProperty]] = None
    """Описание функции"""
    required: Optional[List[str]] = None
    """Список обязательных параметров"""



from typing import Optional

from gigachat.models.function_call import FunctionCall
from gigachat.models.messages_role import MessagesRole
from gigachat.pydantic_v1 import BaseModel


class MessagesChunk(BaseModel):
    """Короткое сообщение"""

    role: Optional[MessagesRole] = None
    """Роль автора сообщения"""
    content: Optional[str] = None
    """Текст сообщения"""
    function_call: Optional[FunctionCall] = None
    """Вызов функции"""
